--[[-------------------------------------------------------------------------
-- Simple Configuration Panel for SIA and STA by Saphod - Draenor EU
-- All rights reserved
-------------------------------------------------------------------------]]--

local addonName, addon = ...

-- Create frame for configuration panel
addon.panel = CreateFrame('Frame', addonName .. 'Panel', InterfaceOptionsFramePanelContainer)
addon.panel:Hide()
addon.panel.name = addonName

-- Local shorthands for accessing the configuration and panel
local cfg = addon.cfg
local panel = addon.panel

-- Will hold controls with values
panel.controls = {}

-- Create a unique name for a control
local UniqueName
do
	local controlID = 1

	function UniqueName(name)
		controlID = controlID + 1
		return string.format('%s_%s_%02d', addonName, name, controlID)
	end
end

-- Function called to (re-)load values on show
function panel:ConfigLoad()
	-- Go through the controls of the configuration frame
	for _, control in pairs(self.controls) do
		-- Call the controls function to load it's value
		control.currentValue = control.LoadValue()

		-- Call the update function to make it reflect the change
		control:UpdateValue()
	end
end

-- Function called when Okay button is pressed on configuration panel
function panel:ConfigOkay()
	-- Go through the controls of the configuration frame
	for _, control in pairs(self.controls) do
		-- Call the controls function to save it's value
		control.SaveValue(control.currentValue)
	end
end

-- Function called when Defaults button is pressed on configuration panel
function panel:ConfigDefault()
	-- Go through the controls of the configuration frame
	for _, control in pairs(self.controls) do
		-- Set the controls value to the default
		control.currentValue = control.defaultValue
	end
end

-- Function called to refresh values
function panel:ConfigRefresh()
	-- Go through the controls of the configuration frame
	for _, control in pairs(self.controls) do
		-- Call the update function to make it reflect any change
		control:UpdateValue()
	end
end

-- Initialize configuration panel
function panel:Initialize(text, subText)
	-- Add a title heading at the top
	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	title:SetPoint('TOPLEFT', 16, -16)
	title:SetText(text)

	-- Add a subtitle heading just below
	local subTitle = self:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	subTitle:SetPoint('TOPLEFT', title, 'BOTTOMLEFT', 0, -8)
	subTitle:SetText(subText)

	-- Add methods called by the frame
	self.okay = self.ConfigOkay
	self.default = self.ConfigDefault
	self.refresh = self.ConfigRefresh

	-- Reload all values each time the configuration panel is shown
	-- Note: This is a compromize to allow the slahs commands to change the 
	-- values, while still making the cancel and default buttons work as
	-- expected. Alternatively we could have the slash command handler
	-- notify the control panel if it changes values.
	self:SetScript('OnShow', self.ConfigLoad)

	-- Add out panel to the interface options addons menu
	InterfaceOptions_AddCategory(self)
end

-- Create a heading in the configuration panel
function panel:CreateHeading(text)
	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLeft')
	title:SetText(text)

	return title
end

-- Create a checkbox in the configuration panel
function panel:CreateCheckBox(text, LoadValue, SaveValue, defaultValue)
	local checkBox = CreateFrame('CheckButton', UniqueName('CheckButton'), self, 'ChatConfigCheckButtonTemplate')

	checkBox.LoadValue = LoadValue
	checkBox.SaveValue = SaveValue
	checkBox.defaultValue = defaultValue
	checkBox.UpdateValue = function(self) self:SetChecked(self.currentValue) end

	getglobal(checkBox:GetName() .. 'Text'):SetText(text)

	-- Update currentValue on click
	checkBox:SetScript('OnClick', function(self) self.currentValue = self:GetChecked() end)

	self.controls[checkBox:GetName()] = checkBox

	return checkBox
end

-- Function used by each 'button' of the dropdown to select it
local function DropDownSetValue(self, arg1, arg2, checked)
	-- arg1 is the DropDown frame, arg2 is the selected value
	arg1.currentValue = arg2
	UIDropDownMenu_SetSelectedValue(arg1, arg2)
	UIDropDownMenu_SetText(arg1, arg2)
end

-- Initialization function for dropdowns, adds all values in valueList
local function DropDownInitialize(frame, level, menuList)
	local info = UIDropDownMenu_CreateInfo()

	-- Add all values in valueList to dropdown
	for _, v in ipairs(frame.valueList) do
		info.text = v
		info.value = v
		info.checked = false
		info.func, info.arg1, info.arg2 = DropDownSetValue, frame, v
		UIDropDownMenu_AddButton(info)
	end
end

-- Create a simple dropdown in the configuration panel
function panel:CreateDropDown(text, valueList, LoadValue, SaveValue, defaultValue)
	local dropDown = CreateFrame('Frame', UniqueName('DropDown'), self, 'UIDropDownMenuTemplate')

	-- Add a title at the top left corner of the dropdown
	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmallLeft')
	title:SetText(text)
	title:SetPoint('BOTTOMLEFT', dropDown, 'TOPLEFT', 16, 0)

	UIDropDownMenu_JustifyText(dropDown, 'LEFT')
	UIDropDownMenu_SetWidth(dropDown, 100)

	dropDown.LoadValue = LoadValue
	dropDown.SaveValue = SaveValue
	dropDown.defaultValue = defaultValue
	dropDown.UpdateValue = function(self)
		UIDropDownMenu_SetSelectedValue(self, self.currentValue)
		UIDropDownMenu_SetText(self, self.currentValue)
	end

	-- Add values in valueList to dropdown
	dropDown.valueList = valueList
	UIDropDownMenu_Initialize(dropDown, DropDownInitialize)

	self.controls[dropDown:GetName()] = dropDown

	return dropDown
end

-- Initialize and add configuration panel
panel:Initialize(addonName, addon.version)

-- Add configuration panel controls
local controlCheckBoxEnabled = panel:CreateCheckBox(
	'Enable announcing',
	function() return cfg.isEnabled end,
	function(v) if v then addon:Enable() else addon:Disable() end end,
	true)
controlCheckBoxEnabled:SetPoint('TOPLEFT', 16, -76)

local controlHeadingSolo = panel:CreateHeading('When Solo')
controlHeadingSolo:SetPoint('TOPLEFT', controlCheckBoxEnabled, 'BOTTOMLEFT', 0, -32)

local controlDropDownSoloOwn = panel:CreateDropDown(
	'Own to',
	{'off', 'self', 'say'},
	function() return cfg.solo.own end,
	function(v) cfg.solo.own = v end,
	'self')
controlDropDownSoloOwn:SetPoint('TOPLEFT', controlHeadingSolo, 'BOTTOMLEFT', 0, -32)

local controlHeadingGroup = panel:CreateHeading('When in a Group')
controlHeadingGroup:SetPoint('TOPLEFT', controlDropDownSoloOwn, 'BOTTOMLEFT', 0, -32)

local controlDropDownGroupOwn = panel:CreateDropDown(
	'Own to',
	{'off', 'self', 'say', 'party'},
	function() return cfg.party.own end,
	function(v) cfg.party.own = v end,
	'self')
controlDropDownGroupOwn:SetPoint('TOPLEFT', controlHeadingGroup, 'BOTTOMLEFT', 0, -32)

local controlDropDownGroupOther = panel:CreateDropDown(
	'Other to',
	{'off', 'self', 'say', 'party'},
	function() return cfg.party.other end,
	function(v) cfg.party.other = v end,
	'self')
controlDropDownGroupOther:SetPoint('TOPLEFT', controlDropDownGroupOwn, 'TOPRIGHT', 32, 0)

local controlHeadingRaid = panel:CreateHeading('When in a Raid')
controlHeadingRaid:SetPoint('TOPLEFT', controlDropDownGroupOwn, 'BOTTOMLEFT', 0, -32)

local controlDropDownRaidOwn = panel:CreateDropDown(
	'Own to',
	{'off', 'self', 'say', 'party', 'raid'},
	function() return cfg.raid.own end,
	function(v) cfg.raid.own = v end,
	'say')
controlDropDownRaidOwn:SetPoint('TOPLEFT', controlHeadingRaid, 'BOTTOMLEFT', 0, -32)

local controlDropDownRaidOther = panel:CreateDropDown(
	'Other to',
	{'off', 'self', 'say', 'party', 'raid'},
	function() return cfg.raid.other end,
	function(v) cfg.raid.other = v end,
	'self')
controlDropDownRaidOther:SetPoint('TOPLEFT', controlDropDownRaidOwn, 'TOPRIGHT', 32, 0)
